/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/09/04 19:18:28
 */

import { nextTick } from 'wkit'

const APP_CONFIG_DIR = `${native.env.CONFIG_DIR}/hosts-switch`
const HOST_FILE = `${APP_CONFIG_DIR}/host.cache`
const LOCK_FILE = `${APP_CONFIG_DIR}/lock`

export function noop() {}

export function checkPermission() {
  return native.fs.access('/etc/hosts', 'a+')
}

export async function getHistory() {
  if (await native.fs.isfile(LOCK_FILE)) {
    let cache = await native.fs.read(HOST_FILE)
    return JSON.parse(cache)
  }

  let cache = await native.fs.read('/etc/hosts')
  let records = cache.split(/[\n\r]+/)
  let list = []
  let dict = {}

  records.forEach(str => {
    str = str.trim()
    let matches = str.match(/^(#*?)\s*(\d+\.\d+\.\d+\.\d+)\s+(.*)/)

    if (matches) {
      let names = matches[3].split(/\s+/).map(it => it.trim())
      let name
      while ((name = names.pop())) {
        list.push({ ip: matches[2], enabled: !matches[1], name })
      }
    }
  })
  records = null

  list.forEach(it => {
    it.name = it.name.split('.')
    let domain = it.name.splice(-2, 2).join('.')
    if (domain === 'com.cn' || domain === 'org.cn' || domain === 'net.cn') {
      domain = it.name.pop() + '.' + domain
    }

    if (dict[domain]) {
      dict[domain].push({
        value: it.ip,
        enabled: it.enabled,
        record: it.name.join('.') || '@',
        remark: ''
      })
    } else {
      dict[domain] = [
        {
          value: it.ip,
          enabled: it.enabled,
          record: it.name.join('.') || '@',
          remark: ''
        }
      ]
    }
  })
  list = null
  try {
    await native.fs.write(HOST_FILE, JSON.stringify(dict))
    await native.fs.write(LOCK_FILE, '')
  } catch (err) {}
  return dict
}

export function saveHosts(dict) {
  nextTick(async () => {
    var txt = ''
    for (let k in dict) {
      for (let it of dict[k]) {
        if (it.enabled) {
          var name = it.record === '@' ? '' : it.record
          if (name) {
            name += '.'
          }
          txt += `${it.value.padEnd(15, ' ')}    ${name + k}\n`
        }
      }
      txt += '\n'
    }

    try {
      await native.fs.write(HOST_FILE, JSON.stringify(dict))
      await native.fs.write('/etc/hosts', txt)
    } catch (err) {
      layer.alert(err)
    }
  })
}
